﻿namespace Ecng.Trading.Xaml
{
	using System;
	using System.Windows.Threading;

	using Ecng.Collections;
	using Ecng.Trading.Algo.Logging;
	using Ecng.Trading.Algo.Strategies;
	using Ecng.Xaml;

	/// <summary>
	/// Логгер стратегии, записывающий данные в визуальное окно <see cref="LogWindow"/> или <see cref="LogControl"/>.
	/// </summary>
	public class GuiStrategyLogger : StrategyLogger
	{
		private readonly LogWindow _window;
		private readonly LogControl _control;
		private readonly SynchronizedDictionary<Strategy, LogWindow> _windows;

		/// <summary>
		/// Создать логгер. Для каждой стратегии будет отдельно создаваться собственное окно <see cref="LogWindow"/>.
		/// </summary>
		public GuiStrategyLogger()
		{
			_windows = new SynchronizedDictionary<Strategy, LogWindow>();
		}

		/// <summary>
		/// Создать логгер.
		/// </summary>
		/// <param name="window">Визуальное окно, в которое будут записываться логи.</param>
		public GuiStrategyLogger(LogWindow window)
		{
			if (window == null)
				throw new ArgumentNullException("window");

			_window = window;
		}

		/// <summary>
		/// Создать логгер.
		/// </summary>
		/// <param name="control">Визуальный контрол, в который будут записываться логи.</param>
		public GuiStrategyLogger(LogControl control)
		{
			if (control == null)
				throw new ArgumentNullException("control");

			_control = control;
		}

		/// <summary>
		/// Записать сообщение, которое сгенерировала переданная стратегия.
		/// </summary>
		/// <param name="strategy">Стратегия, которая сгенерировала сообщение.</param>
		/// <param name="state">Тип сообщения.</param>
		/// <param name="message">Сообщение.</param>
		public override void WriteMessage(Strategy strategy, StrategyErrorStates state, string message)
		{
			Dispatcher.CurrentDispatcher.GuiSync(() =>
			{
				var entry = new LogEntry
				{
					ErrorState = state,
					Message = message,
					Time = DateTime.Now,
					Strategy = strategy,
				};

				if (_control != null)
					_control.Entries.Add(entry);
				else
				{
					var wnd = _window ?? _windows.SafeAdd(strategy, s =>
					{
						var w = new LogWindow { Title = s.ToString() };
						w.Show();
						return w;
					});

					wnd.Entries.Add(entry);	
				}
			});
		}
	}
}