﻿namespace Ecng.Trading.Xaml
{
	using System;
	using System.Collections.Generic;
	using System.Collections.ObjectModel;
	using System.Linq;

	using Ecng.Collections;
	using Ecng.Trading.Algo.Logging;
	using Ecng.Trading.Algo.Strategies;
	using Ecng.Xaml;

	using Hardcodet.Wpf.GenericTreeView;

	/// <summary>
	/// Окно для мониторинга работы торговых стратегий.
	/// </summary>
	public partial class StrategiesMonitorWindow
	{
		private sealed class NodeLogInfo
		{
			public NodeLogInfo(LogControl ctrl, StrategyLogger logger)
			{
				if (ctrl == null)
					throw new ArgumentNullException("ctrl");

				if (logger == null)
					throw new ArgumentNullException("logger");

				this.Ctrl = ctrl;
				this.Logger = logger;
			}

			public LogControl Ctrl { get; private set; }
			public StrategyLogger Logger { get; private set; }
		}

		private sealed class RootStrategy : Strategy
		{
			protected override StrategyProcessResults OnProcess()
			{
				return StrategyProcessResults.Stop;
			}
		}

		private readonly Dictionary<StrategyNode, NodeLogInfo> _logInfo = new Dictionary<StrategyNode, NodeLogInfo>();
		private readonly StrategyLogger _rootLogger;
		private readonly StrategyNode _rootNode;

		/// <summary>
		/// Создать <see cref="StrategiesMonitorWindow"/>.
		/// </summary>
		public StrategiesMonitorWindow()
		{
			InitializeComponent();

			_rootNode = new StrategyNode(new RootStrategy { Name = "Менеджер стратегий" }, this);
			_rootLogger = new GuiStrategyLogger(_rootLogControl);
			_logInfo.Add(_rootNode, new NodeLogInfo(_rootLogControl, _rootLogger));

			_strategiesTree.ObserveChildItems = true;
			_strategiesTree.Items = new ObservableCollection<StrategyNode> { _rootNode };
		}

		private StrategyManager _strategyManager;

		/// <summary>
		/// Менеджер управления торговыми стратегиями <see cref="Strategy"/>.
		/// </summary>
		public StrategyManager StrategyManager
		{
			get { return _strategyManager; }
			set
			{
				if (value != _strategyManager)
				{
					if (_strategyManager != null)
					{
						_strategyManager.StrategyRegistered -= OnStrategyRegistered;
						_strategyManager.StrategyUnRegistered -= OnStrategyUnRegistered;
					}

					_rootNode.ChildNodes.Clear();

					_strategyManager = value;

					if (_strategyManager != null)
					{
						_strategyManager.StrategyRegistered += OnStrategyRegistered;
						_strategyManager.StrategyUnRegistered += OnStrategyUnRegistered;

						_strategyManager.Strategies.ForEach(AddNode);
						_strategiesTree.Refresh();
					}
				}
			}
		}

		private void OnStrategyUnRegistered(Strategy strategy)
		{
			this.GuiSync(() =>
			{
				var node = _rootNode.ChildNodes.First(n => n.Strategy == strategy);
				_rootNode.ChildNodes.Remove(node);
				RemoveLogControl(node);

				_strategiesTree.Refresh();
			});
		}

		private void OnStrategyRegistered(Strategy strategy)
		{
			this.GuiSync(() =>
			{
				AddNode(strategy);
				_strategiesTree.Refresh();
			});
		}

		private void AddNode(Strategy strategy)
		{
			var node = new StrategyNode(strategy, this);
			_rootNode.ChildNodes.Add(node);
			AddLogControl(node);
		}

		internal void AddLogControl(StrategyNode node)
		{
			if (node == null)
				throw new ArgumentNullException("node");

			var ctrl = new LogControl();
			var logger = new GuiStrategyLogger(ctrl);
			logger.Strategies.Add(node.Strategy);
			this.RootGrid.Children.Add(ctrl);
			ctrl.SetVisibility(false);
			ctrl.SetColumn(1);

			_logInfo.Add(node, new NodeLogInfo(ctrl, logger));

			_rootLogger.Strategies.Add(node.Strategy);
		}

		internal void RemoveLogControl(StrategyNode node)
		{
			var info = _logInfo[node];
			info.Logger.Dispose();
			this.RootGrid.Children.Remove(info.Ctrl);
			_logInfo.Remove(node);
			node.Dispose();
			_rootLogger.Strategies.Remove(node.Strategy);
		}

		private void OnSelectedItemChanged(object sender, RoutedTreeItemEventArgs<StrategyNode> e)
		{
			if (e.NewItem != null)
				_logInfo[e.NewItem].Ctrl.SetVisibility(true);
			else
				_rootLogControl.SetVisibility(true);

			if (e.OldItem != null)
			{
				var info = _logInfo.TryGetValue(e.OldItem);

				if (info != null)
					info.Ctrl.SetVisibility(false);
			}
			else
				_rootLogControl.SetVisibility(false);
		}
	}
}